import { BiFunctor, Monad, PatternMatch, Setoid } from "../interfaces";
import Maybe from "./maybe";
declare abstract class Either<A, B> implements Setoid, BiFunctor<A, B>, Monad<B>, PatternMatch {
    /**
     * @function Either.of
     * @constructor
     * @memberof Either
     * @param {any} v - Value
     * @description Creates a Right Either
     */
    static of<A, B>(v: B): Either<A, B>;
    /**
     * @function Either.Right
     * @constructor
     * @memberof Either
     * @param {any} v - Value
     * @description Creates a Right Either
     */
    static Right<A, B>(v: B): Either<A, B>;
    /**
     * @function Either.Left
     * @constructor
     * @memberof Either
     * @param {any} v - Value
     * @description Creates a Left Either
     */
    static Left<A, B>(v: A): Either<A, B>;
    /**
     * @function Either.fromNullable
     * @constructor
     * @memberof Either
     * @param {any} v - Value
     * @description Creates a Right if the value is not null or undefiend else creates a Left
     */
    static fromNullable<A, B>(v: any): Either<A, B>;
    /**
     * @function Either.withDefault
     * @constructor
     * @memberof Either
     * @param {any} def - Value
     * @param {any} v - Value
     * @description Creates a Right if the value v is not null or undefiend else creates a Right with the default value def
     */
    static withDefault<A, B>(def: any, v: any): Either<A, B>;
    /**
     * @function Either.swap
     * @memberof Either
     * @description Swap the Left and Right elements of the current Either
     */
    static swap<A, B>(e: Either<A, B>): Either<A, B>;
    /**
     * @function Either.try
     * @memberof Either
     * @param {Function} f - A function that may throw an error
     * @description * Executes the passed function that may throw and converts it to an Either type.
     */
    static try(f: Function): <A, B>(...args: any[]) => Either<A, B>;
    /**
     * @function Either.bimap
     * @memberof Either
     * @param {any} e - Etiher type
     * @param {Function} fl - Function to be applied on the Left element
     * @param {Function} fr - Function to be applied on the Right element
     * @description A static method that applies fl to the Left element or fr to the Right element of the current Either
     */
    static bimap<A, B, C, D>(e: Either<A, B>, fl: (a: A) => C, fr: (a: B) => D): Either<C, D>;
    /**
     * @function Either.isLeft
     * @memberof Either
     * @description An static method that returns true if the current Either is a Left
     */
    static isLeft<A, B>(v: Either<A, B>): boolean;
    /**
     * @function Either.isRight
     * @memberof Either
     * @description An static method that returns true if the current Either is a Right
     */
    static isRight<A, B>(v: Either<A, B>): boolean;
    protected _value: A | B;
    /**
     * @function Either.equals
     * @memberof Either
     * @param {any} n - Any Value of Type Setoid
     * @description Returns true if the current and the passed element are of Either type with the same value
     */
    abstract equals(n: Setoid): boolean;
    /**
     * @function Either.map
     * @memberof Either
     * @param {Function} f - Function
     * @description Applies the passed function to the value of the current Either if it is a Just
     */
    abstract map<C>(f: (a: B) => C): Either<A, C>;
    /**
     * @function Either.bimap
     * @memberof Either
     * @param {Function} fl - Function to be applied on the Left element
     * @param {Function} fr - Function to be applied on the Right element
     * @description Apply fl to the Left element or fr to the Right element of the current Either
     */
    abstract bimap<C, D>(fl: (a: A) => C, fr: (a: B) => D): Either<C, D>;
    /**
     * @function Either.chain
     * @memberof Either
     * @param {Function} f - Function that returns another Either
     * @description An instance method that can chain together many computations that return a Either type
     */
    abstract chain<C>(f: (a: B) => Either<A, C>): Either<A, C>;
    abstract caseOf(o: {
        Left: Function;
    } | {
        Right: Function;
    }): any;
    /**
     * @function Either.swap
     * @memberof Either
     * @description Swap the Left and Right elements of the current Either
     */
    abstract swap(): Either<A, B>;
    /**
     * @function Either.isLeft
     * @memberof Either
     * @description An instance method that returns true if the current Either is a Left
     */
    abstract isLeft(): boolean;
    /**
     * @function Either.isRight
     * @memberof Either
     * @description An instance method that returns true if the current Either is a Right
     */
    abstract isRight(): boolean;
    of<C>(v: C): Either<A, C>;
    /**
     * @function Either.ap
     * @memberof Either
     * @param {any} j - Either with a function
     * @description Applies the function inside the passed Either to the current Either if it is a Right
     */
    ap<C>(j: Either<A, (a: B) => C>): Either<B, C>;
    /**
     * @function Either.getValue
     * @memberof Either
     * @description Get the value within the Either
     */
    getValue(): A | B;
}
export default Either;
/**
 * @function maybeToEither
 * @param {Maybe} m - Maybe type
 * @description Converts a Maybe type to an Either Type
 */
export declare const maybeToEither: <A>(m: Maybe<A>) => any;
