import { Alt, Monad, PatternMatch, Setoid } from "../interfaces";
import Either from "./either";
export default abstract class Maybe<A> implements Setoid, Monad<A>, Alt<A>, PatternMatch {
    /**
     * @function Maybe.of
     * @constructor
     * @memberof Maybe
     * @param {any} v - Value
     * @description Creates a Just
     */
    static of<A>(v: A): Maybe<A>;
    /**
     * @function Maybe.zero
     * @constructor
     * @memberof Maybe
     * @description Creats a Nothing
     */
    static zero<A>(): Maybe<A>;
    /**
     * @function Maybe.Just
     * @constructor
     * @memberof Maybe
     * @param {any} v - Value
     * @description Creates a Just
     */
    static Just<A>(v: A): Maybe<A>;
    /**
     * @function Maybe.Nothing
     * @constructor
     * @memberof Maybe
     * @description Creats a Nothing
     */
    static Nothing<A>(): Maybe<A>;
    /**
     * @function Maybe.fromNullable
     * @constructor
     * @memberof Maybe
     * @param {any} v - Value
     * @description Creates a Just if the value is not null or undefiend else creates a Nothing
     */
    static fromNullable<A>(v: any): Maybe<A>;
    /**
     * @function Maybe.withDefault
     * @constructor
     * @memberof Maybe
     * @param {any} def - Value
     * @param {any} v - Value
     * @description Creates a Just if the value v is not null or undefiend else creates a Just with the default value def
     */
    static withDefault<A>(def: any, v: any): Maybe<A>;
    /**
     * @function Maybe.catMaybes
     * @memberof Maybe
     * @param {Array<any>} ar - Array of Maybes
     * @description A static method that takes an Array of Maybes and returns back an Array of values of all the Just in the passed Array
     */
    static catMaybes<A>(ar: Array<Maybe<A>>): Array<any>;
    /**
     * @function Maybe.isNothing
     * @memberof Maybe
     * @param {any} v - Maybe
     * @description A static method that returns true if the passed Maybe is a Nothing
     */
    static isNothing<A>(v: Maybe<A>): boolean;
    /**
     * @function Maybe.isJust
     * @memberof Maybe
     * @param {any} v - Maybe
     * @description A static method that returns true if the passed Maybe is a Just
     */
    static isJust<A>(v: Maybe<A>): boolean;
    protected _value: A;
    /**
     * @function Maybe.equals
     * @memberof Maybe
     * @param {any} n - Any Value of Type Setoid
     * @description Returns true if the current and the passed element are of Maybe type with the same value
     */
    abstract equals(n: Setoid): boolean;
    /**
     * @function Maybe.map
     * @memberof Maybe
     * @param {Function} f - Function
     * @description Applies the passed function to the value of the current Maybe if it is a Just
     */
    abstract map<B>(f: (a: A) => B): Maybe<B>;
    /**
     * @function Maybe.chain
     * @memberof Maybe
     * @param {Function} f - Function that returns another Maybe
     * @description An instance method that can chain together many computations that return a Maybe type
     */
    abstract chain<B>(f: (a: A) => Maybe<B>): Maybe<B>;
    abstract caseOf(o: {
        Nothing: Function;
    } | {
        Just: Function;
    }): any;
    /**
     * @function Maybe.isNothing
     * @memberof Maybe
     * @description An instance method that returns true if the current Maybe is a Nothing
     */
    abstract isNothing(): boolean;
    /**
     * @function Maybe.isJust
     * @memberof Maybe
     * @description An instance method that returns true if the current Maybe is a Nothing
     */
    abstract isJust(): boolean;
    of<B>(v: B): Maybe<B>;
    /**
     * @function Maybe.alt
     * @memberof Maybe
     * @param {any} v - Maybe
     * @description An instance method that returns the current maybe if it is a Just else returns the passed Maybe
     */
    alt<B>(v: Maybe<B>): Maybe<B>;
    /**
     * @function Maybe.ap
     * @memberof Maybe
     * @param {any} j - Maybe with a function
     * @description Applies the function inside the passed Maybe to the current Maybe if it is a Just
     */
    ap<B>(j: Maybe<(a: A) => B>): Maybe<B>;
    /**
     * @function Maybe.getValue
     * @memberof Maybe
     * @description Get the value within the Maybe
     */
    getValue(): A;
}
export declare class Nothing<A> extends Maybe<A> {
    equals(n: Setoid): boolean;
    map<B>(f: (a: A) => B): Maybe<B>;
    chain<B>(f: (a: A) => Maybe<B>): Maybe<B>;
    isJust(): boolean;
    isNothing(): boolean;
    toString(): string;
    caseOf(o: {
        Nothing: Function;
    }): any;
}
export declare class Just<A> extends Maybe<A> {
    constructor(v: any);
    equals(j: Setoid): boolean;
    map<B>(f: (a: A) => B): Maybe<B>;
    chain<B>(f: (a: A) => Maybe<B>): Maybe<B>;
    isJust(): boolean;
    isNothing(): boolean;
    toString(): string;
    caseOf(o: {
        Just: Function;
    }): any;
}
/**
 * @function eitherToMaybe
 * @param {Maybe} m - Maybe type
 * @description Converts a Either type to an Maybe Type
 */
export declare const eitherToMaybe: <A, B>(e: Either<A, B>) => any;
/**
 * @function prop
 * @param {string | number} key - Key
 * @param {Object} o - Object
 * @description Returns a Maybe Just if value exists for the given key else returns a Nothing
 */
export declare const prop: Function;
