import { BiFunctor, Monad, Semigroup, Setoid } from "../interfaces";
declare class Pair<A, B> implements Setoid, Semigroup, BiFunctor<A, B>, Monad<B> {
    /**
     * @function Pair.of
     * @constructor
     * @param {any} v - Element that will be stored as first and second element of the Pair
     * @description Pair constructor
     */
    static of<A>(v: A): Pair<A, A>;
    private _value;
    /**
     * @function Pair
     * @constructor
     * @param {any} v1 - First element of the Pair
     * @param {any} v2 - second element of the Pair
     * @description Pair constructor
     */
    constructor(v1: A, v2: B);
    of<C>(v: C): Pair<C, C>;
    /**
     * @function Pair.equals
     * @memberof Pair
     * @param {Pair} j - The Pair to compare with
     * @description Check if the values of the current pair is equal to the values of the passed Pair. Pairs are equal only if the first and second values of both Pairs are equal
     */
    equals(j: Pair<A, B>): boolean;
    /**
     * @function Pair.concat
     * @memberof Pair
     * @param {Pair} p - Pair to concat
     * @description Concat the current pair with the passed one. Note that both first and second elements of the Pair should be of type semigroup for cancat to work
     */
    concat(p: Pair<Semigroup, Semigroup>): Pair<A, B>;
    /**
     * @function Pair.fst
     * @memberof Pair
     * @description Get the first element of the Pair
     */
    fst(): A;
    /**
     * @function Pair.snd
     * @memberof Pair
     * @description Get the second element of the Pair
     */
    snd(): B;
    /**
     * @function Pair.ap
     * @memberof Pair
     * @param {Pair} j - Pair with function as the second element
     * @description Applies the function inside the second element of the passed Pair to the current Pair and concats the first element of the second Pair to the first element of the current Pair
     */
    ap<C, D>(j: Pair<C, (b: B) => D>): Pair<Semigroup, D>;
    /**
     * @function Pair.getValue
     * @memberof Pair
     * @description Get the values within the Pair as an Array of length 2
     */
    getValue(): (A | B)[];
    /**
     * @function Pair.map
     * @memberof Pair
     * @param {Function} f - Function
     * @description Apply the function to the second element of the current Pair
     */
    map<C>(f: (a: B) => C): Pair<A, C>;
    /**
     * @function Pair.bimap
     * @memberof Pair
     * @param {Function} f1 - Function to be applied on the first element
     * @param {Function} f2 - Function to be applied on the second element
     * @description Apply f1 to the first element and f2 to the second element of the current Pair
     */
    bimap<C, D>(f1: (a: A) => C, f2: (a: B) => D): Pair<C, D>;
    /**
     * @function Pair.chain
     * @memberof Pair
     * @param {Function} f - Function that returns another Pair
     * @description Chain together many computations that return a Pair
     */
    chain<C, D>(f: (a: B) => Pair<C, D>): Pair<C, D>;
    /**
     * @function Pair.swap
     * @memberof Pair
     * @description Swap the elements of the current pair
     */
    swap(): Pair<A | B, A | B>;
    /**
     * @function Pair.toString
     * @memberof Pair
     * @description Get a stringified version of the Pair
     */
    toString(): string;
}
export default Pair;
