import { Monad } from "../interfaces";
declare class Reader<A, B> implements Monad<B> {
    /**
     * @function Reader.of
     * @constructor
     * @param {any} v - Any
     * @description Reader constructor
     */
    static of<A>(v: A): Reader<{}, {}>;
    private _value;
    /**
     * @function Reader
     * @constructor
     * @param {any} fn - A function of the form (e -> a) that is wrapped by the Reader, nothing is executed until it is run with an environment
     * @description Reader constructor
     */
    constructor(fn: Function);
    of<C>(v: C): Reader<{}, {}>;
    /**
     * @function Reader.ask
     * @memberof Reader
     * @param fn - Optional function to map over the environment
     * @description A construction helper that returns a Reader. ask can take a function, that can be used to map the environment to a different type or value.
     */
    ask(fn?: Function): Reader<A, B>;
    runWith(ctx: A): any;
    /**
     * @function Reader.ap
     * @memberof Reader
     * @param {Reader} j - Reader with function as the second element
     * @description ap allows for values wrapped in a Reader to be applied to functions also wrapped in a Reader. In order to use ap, the Reader must contain a function as its value.
     */
    ap<C, D>(j: Reader<C, (b: B) => D>): Reader<C, D>;
    /**
     * @function Reader.getValue
     * @memberof Reader
     * @description Get the values within the Reader
     */
    getValue(): Function;
    /**
     * @function Reader.map
     * @memberof Reader
     * @param {Function} f - Function
     * @description Apply the function f to the runWith value of the Reader
     */
    map<C>(f: Function): Reader<A, C>;
    /**
     * @function Reader.chain
     * @memberof Reader
     * @param {Function} f - Function that returns another Reader
     * @description Chain together many computations that return a Reader
     */
    chain<C, D>(f: (a: B) => Reader<C, D>): Reader<C, D>;
    /**
     * @function Reader.toString
     * @memberof Reader
     * @description Get a stringified version of the Reader
     */
    toString(): string;
}
export default Reader;
