import { Monad } from "../interfaces";
declare class State<A, B> implements Monad<B> {
    /**
     * @function State.of
     * @constructor
     * @param {any} v - Any
     * @description State constructor
     */
    static of<A, B>(v: A): State<{}, {}>;
    private _value;
    /**
     * @function State
     * @constructor
     * @param {Function} fn
     * @description State constructor
     */
    constructor(fn: Function);
    of<C, D>(v: C): State<{}, {}>;
    /**
     * @function State.get
     * @memberof State
     * @param fn - Optional function to map over the state
     * @description A construction helper that returns a State. get can take a function, that can be used to map the over the state part.
     */
    get(fn?: Function): State<A, B>;
    /**
     * @function State.put
     * @memberof State
     * @param s - An initial resultant state
     * @description A construction helper that returns a State. that when run returns a Pair(result, Unit).
     */
    put(s: B): State<A, B>;
    /**
     * @function State.runWith
     * @memberof State
     * @param s - Initial state
     * @description When called, runWith will run the state transition with the given value as the initial state and will return the resulting Pair (result, state).
     */
    runWith(s: A): any;
    /**
     * @function State.execWith
     * @memberof State
     * @param s - Initial state
     * @description When called, execWith will run the state transition with the given value as the initial state and will return the state.
     */
    execWith(s: A): any;
    /**
     * @function State.evalWith
     * @memberof State
     * @param s - Initial state
     * @description When called, evalWith will run the state transition with the given value as the initial state and will return the resultant.
     */
    evalWith(s: A): any;
    /**
     * @function State.ap
     * @memberof State
     * @param {State} j - State with function as the second element
     * @description Applies the resultant of a given State instance to a function wrapped in another instance. On a State instance that wraps a function, calling ap, providing it another State instance, will return a new State instance with the result of the function in the resultant portion.
     */
    ap<C, D>(j: State<C, (b: B) => D>): State<C, D>;
    /**
     * @function State.getValue
     * @memberof State
     * @description Get the values within the State
     */
    getValue(): Function;
    /**
     * @function State.map
     * @memberof State
     * @param {Function} f - Function
     * @description Apply the function f to the runWith value of the State
     */
    map<C>(f: Function): State<A, C>;
    /**
     * @function State.chain
     * @memberof State
     * @param {Function} f - Function that returns another State
     * @description Chain together many computations that return a State
     */
    chain<C, D>(f: (a: B) => State<C, D>): State<C, D>;
    /**
     * @function State.toString
     * @memberof State
     * @description Get a stringified version of the State
     */
    toString(): string;
}
export default State;
