import { Monad, Semigroup } from "../interfaces";
declare class Task implements Semigroup, Monad<(rej: Function, res: Function) => any> {
    /**
     * @function Task.of
     * @constructor
     * @param {any} v - value that is passed to the resolve function
     * @description Task constructor that creates a Task which immediately resolves
     */
    static of(v: any): Task;
    /**
     * @function Task.rejected
     * @constructor
     * @param {any} v - value that is passed to the rejected function
     * @description Task constructor that creates a Task which immediately gets rejected
     */
    static rejected(v: any): Task;
    private _value;
    /**
     * @function Task
     * @constructor
     * @param {Function} f - A function that is passed with the arguments reject and resolve functions. When reject or resolve functions are called the Task gets rejected or resolved, respectively. Function f normally initiates an asynchronous work or one that has effects, and then, once that completes, either calls the resolve function to resolve the Task or else rejects it.
     * @description Task constructor
     */
    constructor(f: (rej: Function, res: Function) => any);
    of(v: any): Task;
    rejected(v: any): Task;
    /**
     * @function Task.fork
     * @constructor
     * @param {Function} reject - Function to be called when the Task is rejected
     * @param {Function} resolve - Function to be called when the Task is resolved
     * @description Executes the Task
     */
    fork(reject: Function, resolve: Function): void;
    /**
     * @function Task.toString
     * @memberof Task
     * @description Get a stringified version of the Task
     */
    toString(): string;
    /**
     * @function Task.map
     * @memberof Task
     * @param {Function} f - Function
     * @description Apply the function f to value of a successfully resolved Task
     */
    map<A, B>(f: (a: A) => B): Task;
    /**
     * @function Task.getValue
     * @memberof Task
     * @description Get the function within the Task
     */
    getValue(): (rej: Function, res: Function) => any;
    /**
     * @function Task.ap
     * @memberof Task
     * @param {Task} t - Task with function as the second element
     * @description * Applys the successful value of the Task t to the successful value of the current Task
     */
    ap(t: Task): Task;
    /**
     * @function Task.concat
     * @memberof Task
     * @param {Task} t - Task to concat
     * @description Concat the current Task with the passed one and get a new Task. Which when resloved would get the successfull result of both the task.
     */
    concat(t: Task): Task;
    /**
     * @function Task.chain
     * @memberof Task
     * @param {Function} f - Function that returns another Task
     * @description Chain together many computations that return a Task
     */
    chain(f: Function): Task;
    /**
     * @function Task.toPromise
     * @memberof Task
     * @description Converts the current task to a Promise
     */
    toPromise(): Promise<{}>;
}
export default Task;
