/**
 * @function throwError
 * @param {string} x - Error message
 * @description Throw an error
 */
declare const throwError: (x: string) => never;
/**
 * @function id
 * @param {*} x - Any
 * @description Identity Function
 */
declare const id: <A>(x: A) => A;
/**
 * @function constant
 * @param {*} x - Any
 * @description Takes any value and gives back a function that will return that same value no matter what you pass it
 */
declare const constant: <A>(x: A) => () => A;
/**
 * @function isFunction
 * @param {*} f - Any
 * @description Check if supplied argument is
 */
declare const isFunction: (f: any) => boolean;
/**
 * @function once
 * @param {Function} f - Function to be called once
 * @description Create a function which calls the given function only once
 */
declare const once: (f: Function) => (...args: any[]) => any;
/**
 * @function curry
 * @param {Function} f - Function to be curried
 * @description Returns a curried equivalent of the provided function.
 */
declare const curry: (fn: Function) => Function;
/**
 * @function ncurry
 * @param {Function} fn - Function to be curried which accepts a single named argument object
 * @param {Array<string>} args - key names of the Function argument object
 * @description Returns a curried equivalent of the provided function which will accept named arguments in any order
 */
declare const ncurry: (fn: Function, args: string[]) => Function;
/**
 * @function compose
 * @param {...Function} fns - functions to compose
 * @description Performs right-to-left function composition.
 * @summary compose :: Function
 */
declare const compose: (...fns: Function[]) => Function;
declare const fmap: Function;
declare const bimap: Function;
declare const chain: Function;
declare const caseOf: Function;
/**
 * @function liftAn
 * @param {Function} f - Function to be lifted
 * @param {...Apply} fn - Applys to the function
 * @description lets us combine n separate wrapped values into one with a given function.
 */
declare const liftAn: Function;
/**
 * @function liftA1
 * @param {Function} f - Function to be lifted
 * @param {Apply} fn - 'Apply' to the function
 * @description Aka fmap
 */
declare const liftA1: Function;
/**
 * @function liftA2
 * @param {Function} f - Function to be lifted
 * @param {Apply} f1 - 'Apply' to the function
 * @param {Apply} f2 - 'Apply' to the function
 * @description lets us combine 2 separate wrapped values into one with a given function.
 */
declare const liftA2: Function;
/**
 * @function liftA3
 * @param {Function} f - Function to be lifted
 * @param {Apply} f1 - 'Apply' to the function
 * @param {Apply} f2 - 'Apply' to the function
 * @param {Apply} f3 - 'Apply' to the function
 * @description lets us combine 3 separate wrapped values into one with a given function.
 */
declare const liftA3: Function;
/**
 * @function liftA4
 * @param {Function} f - Function to be lifted
 * @param {Apply} f1 - 'Apply' to the function
 * @param {Apply} f2 - 'Apply' to the function
 * @param {Apply} f3 - 'Apply' to the function
 * @param {Apply} f4 - 'Apply' to the function
 * @description lets us combine 4 separate wrapped values into one with a given function.
 */
declare const liftA4: Function;
/**
 * @function liftA2
 * @param {Function} f - Function to be lifted
 * @param {Apply} f1 - 'Apply' to the function
 * @param {Apply} f2 - 'Apply' to the function
 * @param {Apply} f3 - 'Apply' to the function
 * @param {Apply} f4 - 'Apply' to the function
 * @param {Apply} f5 - 'Apply' to the function
 * @description lets us combine 2 separate wrapped values into one with a given function.
 */
declare const liftA5: Function;
/**
 * @function when
 * @param {Function} p - Predicate function
 * @param {Function} f - Function to be run on the input
 * @description Returns a function for which it takes one argument and passes it to the given predicate function. If the predicate is satisfied, f is run with the same argument. If the predicate is not satisfied, the argument is returned as is.
 */
declare const when: Function;
export { id, isFunction, throwError, once, fmap, bimap, chain, caseOf, constant, curry, ncurry, compose, liftAn, liftA1, liftA2, liftA3, liftA4, liftA5, when };
